#
# See the file "LICENSE" for the full license governing this code.
#
require 'pathname'
$LOAD_PATH.unshift(Pathname.new(__FILE__).dirname.parent.parent)
$LOAD_PATH.unshift(Pathname.new(__FILE__).dirname.parent.parent.parent.parent + 'easy_type' + 'lib')
require 'easy_type'
require 'puppet_x/enterprisemodules/oracle/access'
require 'puppet_x/enterprisemodules/oracle/title_parser'
require 'puppet_x/enterprisemodules/oracle/default_sid'

Puppet::Type.newtype(:ora_scheduler_job) do
  include EasyType
  include Puppet_X::EnterpriseModules::Oracle::Access
  extend Puppet_X::EnterpriseModules::Oracle::TitleParser
  include Puppet_X::EnterpriseModules::Oracle::DefaultSid

  desc <<-EOD
  This resource allows you to manage PL/SQL packages in the Oracle database.

      ora_package { 'testuser.pkg_manage_my_objects':
        ensure  => 'present',
        source => '/vagrant/tests/package.sql',
      }

  This puppet definition ensures that the package `testuser.pkg_manage_my_objects` is available
  in the database and that its content matches the content defined in the specified source.

  To decide if the package needs an update, the puppet type compares the content in the database, with
  the content in the source file. This comparison is done insenitive to case, white spacing and used quote's
  (the " or the ` ).

  When you have specified `report_errors => true` (which is the default), the type will fail on PL/SQL
  compilation errors. Packages with compilation errors do however end up in the database. On a second Puppet
  run the won't be updated. Puppet reports a warning these resources though?

  ```
  Warning: package TESTUSER.PKG_MANAGE_MY_OBJECTS@test up-to-date, but contains 4 error(s).
  ```

  EOD

  to_get_raw_resources do
    sql_on_all_database_sids "col auto_drop form a10
                              col enabled form a10
                              col logging_level form a40
                              col restartable form a15
                              select owner
                                   , job_name
                                   , auto_drop
                                   , comments
                                   , enabled
                                   , to_char(end_date,'DD-MM-YYYY HH24:Mi:SS') end_date
                                   , job_class
                                   , job_priority
                                   , job_type
                                   , decode(logging_level,
                                            'OFF', 'DBMS_SCHEDULER.LOGGING_OFF',
                                            'RUNS', 'DBMS_SCHEDULER.LOGGING_RUNS',
                                            'FAILED RUNS', 'DBMS_SCHEDULER.LOGGING_FAILED_RUNS',
                                            'FULL', 'DBMS_SCHEDULER.LOGGING_FULL',
                                            logging_level) as logging_level
                                   , max_runs
                                   , max_failures
                                   , (sysdate + max_run_duration * 86400 - sysdate) as max_run_duration
                                   --, raise_events
                                   , repeat_interval
                                   , restartable
                                   --, restart_on_failure
                                   --, restart_on_recovery
                                   , (sysdate + schedule_limit * 86400 - sysdate) as schedule_limit
                                   , to_char(start_date,'DD-MM-YYYY HH24:Mi:SS') as start_date
                                   --, stop_on_window_close
                                   --, store_output
                              from dba_scheduler_jobs
                             where job_type in ('PLSQL_BLOCK', 'STORED_PROCEDURE')"
  end

  on_create do
    apply
  end

  on_modify do
    nil
  end

  on_destroy do
    [:sql, "exec DBMS_SCHEDULER.DROP_JOB(job_name => '#{owner}.#{job_name}', force => TRUE)", {:sid => sid}]
  end

  def apply
    options = { :sid => sid, :parse => false, :catch_extra_errors => /compilation errors/ }
    options[:failonsqlfail] = true
    job_action_content = get_content(job_action)
    statement = template('ora_config/ora_scheduler_job/create_job.sql.erb', binding)

    output = sql statement, options
    Puppet.debug(output) if logoutput == :true
    nil
  end

  def get_content(input)
    if Pathname(input).absolute?
      if File.file?(input)
        File.read(input)
      else
        fail "Unable to find file #{input} for job_action content"
      end
    else
      input
    end
  end

  map_title_to_sid(:owner, :job_name) { /(.+)\.(.+)/ }

  ensurable

  parameter :name
  parameter :owner
  parameter :job_name

  parameter :sid
  parameter :logoutput
  parameter :report_errors

  property  :auto_drop
  property  :comments
  property  :enabled
  property  :end_date
  property  :job_action
  property  :job_class
  property  :job_priority
  property  :job_type
  property  :logging_level
  property  :max_runs
  property  :max_failures
  property  :max_run_duration
  # property  :raise_events
  property  :repeat_interval
  property  :restartable
  # property  :restart_on_failure
  # property  :restart_on_recovery
  property  :schedule_limit
  property  :start_date
  # property  :stop_on_window_close
  # property  :store_output

  ora_autorequire(:ora_user, :owner)

end
