#
# See the file "LICENSE" for the full license governing this code.
#
require 'pathname'
$LOAD_PATH.unshift(Pathname.new(__FILE__).dirname.parent.parent)
$LOAD_PATH.unshift(Pathname.new(__FILE__).dirname.parent.parent.parent.parent + 'easy_type' + 'lib')
require 'easy_type'
require 'easy_type/array_property'
require 'puppet_x/enterprisemodules/oracle/access'
require 'puppet_x/enterprisemodules/oracle/title_parser'
require 'puppet/type/ora_tablespace/tablespace_type'
require 'puppet_x/enterprisemodules/oracle/default_sid'
require 'puppet_x/enterprisemodules/oracle/information'
require 'puppet_x/enterprisemodules/oracle/datafile_property'

Puppet::Type.newtype(:ora_tablespace) do
  include EasyType
  include Puppet_X::EnterpriseModules::Oracle::Access
  extend Puppet_X::EnterpriseModules::Oracle::TitleParser
  include Puppet_X::EnterpriseModules::Oracle::DefaultSid
  include Puppet_X::EnterpriseModules::Oracle::Information
  include Puppet_X::EnterpriseModules::Oracle::DatafileProperty

  desc <<-EOD
    This type allows you to manage an Oracle tablespace.
    The datafile and num_datafiles properties are mutually exclusive.
    When specifying multiple datafiles(or num_datfiles > 1) all datafiles will get the same settings for size, autoextend, next and max_size.

    It recognises most of the options that [CREATE TABLESPACE](http://docs.oracle.com/cd/B28359_01/server.111/b28310/tspaces002.htm#ADMIN11359) supports.

        ora_tablespace {'my_app_ts@sid':
          ensure                    => present,
          datafile                  => 'my_app_ts.dbf',
          size                      => 5G,
          logging                   => yes,
          autoextend                => on,
          next                      => 100M,
          max_size                  => 20G,
          extent_management         => local,
          segment_space_management  => auto,
          encryption                => 'AES256',
        }

    Tablespace with multiple datafiles:

        ora_tablespace {'my_app_ts@sid':
          ensure                    => present,
          datafile                  => ['my_app_ts01.dbf', 'my_app_ts02.dbf'],
          size                      => 5G,
          logging                   => yes,
          autoextend                => on,
          next                      => 100M,
          max_size                  => 20G,
          extent_management         => local,
          segment_space_management  => auto,
          encryption                => 'AES256',
        }

    Tablespace with multiple datafiles using Oracle Managed Files:

        ora_tablespace {'my_app_ts@sid':
          ensure                    => present,
          num_datafile              => 3,
          size                      => 5G,
          logging                   => yes,
          autoextend                => on,
          next                      => 100M,
          max_size                  => 20G,
          extent_management         => local,
          segment_space_management  => auto,
          encryption                => 'AES256',
        }

    You can also create an undo tablespace:

        ora_tablespace {'my_undots_1@sid':
          ensure                    => present,
          contents                  => 'undo',
        }

    or a temporary tablespace:

        tablespace {'my_temp_ts@sid':
          ensure                    => present,
          datafile                  => 'my_temp_ts.dbf',
          contents                  => 'temporary',
          size                      => 5G,
          autoextend                => on,
          next                      => 100M,
          max_size                  => 20G,
          extent_management         => local,
          segment_space_management  => auto,
        }

  EOD

  ensurable

  to_get_raw_resources do
    command = for_version(
      :'12'     => template('ora_config/ora_tablespace/index12.sql.erb', binding),
      :default  => template('ora_config/ora_tablespace/index.sql.erb', binding)
    )
    sql_on_all_database_sids command
  end

  validate do
    fail 'bigfile tablespaces can only have one datafile' if self['bigfile'] == :yes && (df_array_size(self['datafile']) > 1 || self['num_datafiles'].to_i > 1)
    fail 'can not specify both \'datafile\' and \'num_datafiles\' properties' if datafile && num_datafiles
    if datafile
      fail 'datafile property should be specified as Sting or Array' unless datafile.is_a?(Array)
    end
  end

  on_create do
    command = "create #{ts_type} #{ts_contents} tablespace \"#{tablespace_name}\""
    command << " segment space management #{segment_space_management}" if segment_space_management
    command << " blocksize #{block_size}" if block_size
    command << " #{ts_encryption}" if encryption
    [:sql, command, { :sid => sid }]
  end

  on_modify do
    # Allow individual properties to do their stuff
  end

  on_destroy do
    [:sql, "drop tablespace \"#{tablespace_name}\" including contents and datafiles", { :sid => sid }]
  end

  map_title_to_sid(:tablespace_name)

  parameter :name
  parameter :tablespace_name
  parameter :sid
  parameter :block_size
  parameter :timeout
  parameter :encryption
  parameter :bigfile

  property  :datafile
  property  :num_datafiles
  property  :size
  group(:autoextend_group) do
    property  :autoextend
    property  :next
    property  :max_size
  end
  property  :extent_management
  property  :segment_space_management
  property  :logging
  property  :contents
end
